#include <iostream>
#include <string>
#include <ctime>
#include <random>
#include <sstream>

using namespace std;

const bool DEBUG = false;
const int ROWS = 5;
const int COLS = 5;
const string ICON_WATER = "~";
const string ICON_FIRE_ON_WATER = "o";
const string ICON_FIRE_ON_SHIP = "x";
const int SHIP_LENGTHS[] = {4, 3, 3, 2};

class debug
{
public:
    static void print(string s)
    {
        if (DEBUG == true) cout << s << endl;
    }
};


int total_ships()
{
    int i = sizeof(SHIP_LENGTHS) / sizeof(int);
    return i;
}

int total_hits()
{
    int total = 0;
    for (int i = 0; i < total_ships(); i++)
        total += SHIP_LENGTHS[i];
    return total;
}


int random(int _min, int _max)
{
    int rnd = (int)(rand()/(double)RAND_MAX * (_max - _min + 1)) + _min;
    return rnd;
}


int random2(int _min, int _max)
{
    //int rnd = (int)(rand() / (double)RAND_MAX * (_max - _min + 1)) + _min;
    //return rnd;
    random_device rd;
    default_random_engine eng(rd());
    uniform_int_distribution<int> distr(_min, _max);
    return distr(eng);
}


class Ship
{
public:
    int x1;
    int y1;
    int x2;
    int y2;

public:


    void build_random(int length)
    {
        while(true)
        {
            bool horizontal = random(0, 1) == 1;
            x1 = random(0, COLS - 1);
            y1 = random(0, ROWS - 1);
            x2 = x1;
            y2 = y1;
            if (horizontal)
            {
                x2 = x1 + length - 1;
            }
            else
            {
                y2 = y1 + length - 1;
            }
            if (x2 < COLS && y2 < ROWS) break;
        }
    }

    bool is_vertical()
    {
        bool v = x1 == x2;
        return v;
    }
};


string private_board_cpu[ROWS][COLS];
string scanner_board_p1[ROWS][COLS];

string private_board_p1[ROWS][COLS];
string scanner_board_cpu[ROWS][COLS];

//[0][0]  [0][1] [0][2]...
//[1][0]  [1][1] [1][2]...
//[2][0]...


void fill_with_icon(string board[][COLS], string icon)
{
    for (int r = 0; r < ROWS; r++)
    {
        for (int c = 0; c < COLS; c++)
        {
            board[r][c] = icon;
        }
    }
}

void show_board(string board[][COLS])
{
    debug::print("board size is " + to_string(ROWS) + "x" + to_string(COLS));

    cout << "   ";
    for (int i = 0; i < COLS; i++)
    {
        cout << i + 1 << " ";
    }
    cout << endl;
    cout << " +-";
    for (int i = 0; i < COLS; i++)
    {
        cout << "--";
    }
    cout << endl;

    for (int r = 0; r < ROWS; r++)
    {
        char c = (char)(65 + r);
        cout << c << "| ";
        for (int c = 0; c < COLS; c++)
        {
            cout << board[r][c] << " ";
        }
        cout << endl;
    }
}


void add_ship_to_board(string board[][COLS], Ship s, string ship_icon)
{
    for(int col = s.x1; col <= s.x2; col++)
    {
        for(int row = s.y1; row <= s.y2; row++)
        {
            board[row][col] = ship_icon;
        }
    }
}

bool check_for_availability(Ship s, string board[][COLS])
{
    for(int col = s.x1; col <= s.x2; col++)
    {
        for(int row = s.y1; row <= s.y2; row++)
        {
            if (board[row][col] != ICON_WATER)
            {
                return false;
            }
        }
    }
    return true;
}

void place_ships(string board[][COLS])
{
    for(int i=0; i<total_ships(); i++)
    {
        Ship s;
        bool ok = false;
        do
        {
            s.build_random(SHIP_LENGTHS[i]);
            ok = check_for_availability(s, board);
        }
        while(ok == false);
        add_ship_to_board(board, s, to_string(SHIP_LENGTHS[i]));
    }

}

void remember()
{
    string name;
    cout << "Please, enter your full name: ";
    getline (std::cin,name);
    cout << "Hello, " << name << "!\n";

    vector<string> lv;
    lv.push_back(name);

    for (int i = 0; i < lv.size(); i++)
    {
        cout << i + 1 << ". " << lv[i] << endl;
    }
}


int main()
{
    srand(time(NULL)); // to have different random numbers every time

    fill_with_icon(scanner_board_p1, ICON_WATER);
    fill_with_icon(private_board_p1, ICON_WATER);
    fill_with_icon(scanner_board_cpu, ICON_WATER);
    fill_with_icon(private_board_cpu, ICON_WATER);
    place_ships(private_board_cpu);
    place_ships(private_board_p1);
    if (DEBUG)
    {
        cout << "P1" << endl;
        show_board(private_board_p1);
        cout << "CPU" << endl;
        show_board(private_board_cpu);
    }
    show_board(scanner_board_p1);

    int shots_fired = 0;
    int shots_missed = 0;
    int shots_hit_p1 = 0;
    int shots_hit_cpu = 0;
    int max_hits = total_hits();
    int shot_points = 1;
    int score = 0;
    while(true)
    {
        if (shots_hit_p1 == max_hits || shots_hit_cpu == max_hits)
        {
            break;
        }
        string fire_xy;
        do
        {
            cout << "Fire at: ";
            cin >> fire_xy;
        }
        while(fire_xy.length() != 2);
        if (fire_xy == "00" || fire_xy == "99")
            break;

        //  'b' - 'a' = 50 - 49 = 0
        int row = fire_xy[0] - 'a';
        int col = stoi(fire_xy.substr(1, 1)) - 1;
        debug::print("Fire at " + to_string(row) + "," + to_string(col));

        string icon_result;
        if (scanner_board_p1[row][col] != ICON_WATER)
        {
            icon_result = scanner_board_p1[row][col];
            cout << "location already used" << endl;
        }
        else if (private_board_cpu[row][col] != ICON_WATER)
        {
            icon_result = ICON_FIRE_ON_SHIP;
            shots_hit_p1++;
            shots_fired++;
            cout << "ship was hit!" << endl;
            score += shot_points;
            shot_points++;
        }
        else
        {
            icon_result = ICON_FIRE_ON_WATER;
            shots_fired++;
            cout << "no ship hit :(" << endl;
            shot_points = 1;
        }
        scanner_board_p1[row][col] = icon_result;
        //
        // Gameplay for computer
        //
        while(true)
        {
            int c_x = random(0, COLS - 1);
            int c_y = random(0, ROWS - 1);
            if (scanner_board_cpu[c_y][c_x] == ICON_WATER)
            {
                if (private_board_p1[c_y][c_x] == ICON_WATER)
                {
                    scanner_board_cpu[c_y][c_x] = ICON_FIRE_ON_WATER;
                }
                else
                {
                    scanner_board_cpu[c_y][c_x] = ICON_FIRE_ON_SHIP;
                    shots_hit_cpu++;
                }
                break;
            }
        }


        //
        cout << "P1" << endl;
        show_board(scanner_board_p1);
        cout << "CPU" << endl;
        show_board(scanner_board_cpu);
        cout << "Shot:" << shots_fired << "  hits:" << shots_hit_p1 << "/" << total_hits()
        << "  score: " << score  << "   hits CPU: " << shots_hit_cpu << endl;
    }
    cout << " Game Over!";
    show_board(private_board_cpu);

    return 0;
}